# This file is part of the "FDN tools" project.
# You can use and redistribute it under the terms of the
# GNU General Public License.
# (c) French Data Network 1997-2005
# Authors and contributors:
# - Benjamin Bayart
package FDN::Sql::DBC;

use strict;
use Exporter ;

our @ISA = ( 'Exporter' ) ;

our @EXPORT = qw (  do_connect
		do_disconnect
		);
our @EXPORT_OK = qw ( );

use FDN::Sql::DB_Access;
use FDN::Common::Config;
use FDN::Common::Constantes;
use FDN::Common::Logger;
our %handlers;

END {
	for my $db ( keys %handlers ) {
		error "You forgot to close database $db.";
		do_disconnect($handlers{$db});
	}
}

sub do_connect($) {
	my ( $base ) = @_;
	if ( defined $handlers{$base} ) {
		return $handlers{$base};
	}
	my $cfg = Load_Config('dbc.cfg');
	if ( ! defined $cfg->{$base} ||
	     ref($cfg->{$base} ne 'HASH') ) {
		die "Pas d'acces configure pour la base '$base' sur cette machine.";
	}
	$cfg = $cfg->{$base};
	if ( ! defined $cfg->{'type'} ) {
		$cfg->{'type'} = 'mysql';
	}
	if ( ! defined $cfg->{'name'} ) {
		$cfg->{'name'} = $base;
	}
	if ( ! defined $cfg->{'user'} ) {
		$cfg->{'user'} = $cfg->{'name'};
	}
	if ( ! defined $cfg->{'host'} ) {
		$cfg->{'host'} = $cfg->{'name'};
	}
	if ( ! defined $cfg->{'passwd'} ) {
		die "Manque le mot de passe de la base $base.";
	}
	my $opts = { 'PrintError' =>0 };
	if ( defined $cfg->{'utf8'} && $cfg->{'utf8'} ) {
		$opts->{'utf8'} = $cfg->{'utf8'};
	}
	my $dbh = connect_db (
		$cfg->{'type'},
		$cfg->{'host'},
		$cfg->{'name'},
		$cfg->{'user'},
		$cfg->{'passwd'},
		$opts, # DBI does not interfere with the output
		FALSE ) ;
	if ( $dbh->{'err_db'} ) {
		die "Erreur connexion base:\n".$dbh->{'err_mess'};
	}
	$dbh->{'DBC-name'} = $base;
	$handlers{$base} = $dbh;
	return $dbh;
}

sub do_disconnect($) {
	my ( $dbh ) = @_;
	my $name;
	if ( ! defined $dbh ) {
		error "You did ask to disconnect from an undef thing...";
		return;
	} elsif ( ref($dbh) eq 'HASH' ) {
		$name = $dbh->{'DBC-name'};
		disconnect_db ( $dbh );
	} else {
		$name = $dbh;
		$dbh = $handlers{$name};
		disconnect_db ( $dbh );
	}
	delete $handlers{$name};
}

1;
