package FDN::Sql::Lists;

use strict;
use Exporter;
use FDN::Common::Logger;
use FDN::Sql::Basic;
use FDN::Sql::DB_Access;
use Data::Dumper;
our @ISA = ( 'Exporter' );
our @EXPORT = qw (
	__list_push
	__list_push_new
	__list_pop
	__list_pop_id
	__list_unshift
	__list_unshift_new
	__list_shift
	__list_shift_id
	__list_insert_after
	__list_detach
	__list_get_list
	__list_get_list_n
	__list_get_relative_n
	__list_get_count
);
our @EXPORT_OK = qw (
);
our %EXPORT_TAGS = qw (
);

#$FDN::Sql::Basic::debug=1;
#$FDN::Sql::DB_Access::debug=1;
=encoding utf-8

=head2 Functions

__list_push
__list_pop
__list_insert_after
__list_detach
__list_get_list
__list_get_list_n
__list_get_count
__list_get_list_ids
__list_get_list_ids_n
__list_get_list_ids_n_from


=head2 Descripter

$list = {
	head_table	=> 'MACHIN',
	head_field	=> 'MACHIN_L1',
	head_count	=> 'MACHIN_NB1', # Optionnel
	tail_field	=> 'MACHIN_T1',  # Optionnel
	item_table	=> 'OBJET',
	item_field	=> 'OBJET_NEXT',
	item_up		=> 'OBJET_UP'    # Optionnel
}

+-----------+		+------------+
| MACHIN    |		| OBJET      |
+-----------+		+------------+
| MACHIN_ID |<-|  /---->| OBJET_ID   |<-\
| MACHIN_L1 |--+-/      | OBJET_NEXT |--/
| MACHIN_T1 |  |--------| OBJET_UP   |
+ ...       +		| ...        |
+-----------+		+------------+


=cut

sub __list_get_head($$$) {
	my ( $dbh, $list, $h ) = @_;
	my $tbl_head = $list->{'head_table'};
	my ( $hid, $head );
	if ( ref($h) ) {
		$head = $h;
		$hid = $head->{$tbl_head.'_ID'} if defined $head;
	} else {
		$hid = $h;
		$head = __par_id ($dbh, $tbl_head, $hid);
	}
	if ( ! defined $head ) {
		error "No head for that list.";
		return ( undef, undef );
	}
	return ( $hid, $head );
}

sub __list_get_head_id($$$) {
	my ( $dbh, $list, $h ) = @_;
	my $tbl_head = $list->{'head_table'};
	if ( ref($h) ) {
		return $h->{$tbl_head.'_ID'};
	} else {
		return $h;
	}
}

sub __list_get_info($) {
	my ( $list ) = @_;
	return ( $list->{'head_table'}, $list->{'item_table'}, $list->{'item_field'}, $list->{'head_field'}, $list->{'tail_field'}, $list->{'item_up'}, $list->{'head_count'} );
}

# Moves an existing element at tail of the list (perl-like push)
#
# dbh    : database handler
# list   : list handle
# h      : id of the list's head, or the head itself
# id     : id of current element
#
# return number of element ???
sub __list_push ($$$$) {
	my ($dbh, $list, $h, $id ) = @_;
	return undef if (ref $list ne 'HASH');
	die "Need tail for push" if (! $list->{'tail_field'});
	my ( $tbl_head, $tbl_item, $next, $first, $last, $up, $count ) = __list_get_info($list);
	my ( $hid, $head ) = __list_get_head($dbh, $list, $h);
	return undef if ! defined $head;

	if ( !defined $head->{$first} || $head->{$first} eq -1 ) {
		# List is empty
		__set_tbl_val($dbh, $tbl_head, $first, $hid, do_null_or_char($id));
		$head->{$first} = $id;
	} else {
		__set_tbl_val ($dbh, $tbl_item, $next, $head->{$last}, do_null_or_char($id));
	}
	__set_tbl_val( $dbh, $tbl_item, $up,   $id, do_null_or_char($hid)) if $up;
	__set_tbl_val ($dbh, $tbl_item, $next, $id, do_null_or_char(-1));
	__set_tbl_val ($dbh, $tbl_head, $last, $hid, do_null_or_char($id));
	$head->{$last} = $id;
	if ( $count ) {
		$head->{$count}++;
		__set_tbl_val($dbh, $tbl_head, $count, $hid, do_null_or_char($head->{$count}));
	}
	return __list_get_count($dbh, $list, $head);
}

sub __list_push_new($$$@) {
	my ( $dbh, $list, $h, @infos ) = @_;
	my $id = __new($dbh, $list->{'item_table'}, @infos);
	__list_push($dbh, $list, $h, $id);
	return $id;
}

# pop an element from the end of the list
#
# dbh    : database handler
# list   : list handle
# h      : id of the list's head, or the head itself
#
# return the id of the element
sub __list_pop_id ($$$) {
	my ($dbh, $list, $h ) = @_;
	return undef if (ref $list ne 'HASH');
	die "Need tail for pop" if (! $list->{'tail_field'});
	my ( $tbl_head, $tbl_item, $next, $first, $last, $up, $count ) = __list_get_info($list);
	my ( $hid, $head ) = __list_get_head($dbh, $list, $h);
	return undef if ! defined $head;

	my $item;
	if ( $head->{$last} == $head->{$first} ) {
		if ( $head->{$first} == -1 ) {
			# List is empty.
			return undef;
		} else {
			# List has only one element
			my $res = $head->{$first};
			__set_tbl_val($dbh, $tbl_head, $first, $hid, do_null_or_char(-1));
			__set_tbl_val($dbh, $tbl_head, $last,  $hid, do_null_or_char(-1));
			$head->{$last} = $head->{$first} = -1;
			if ( $count ) {
				$head->{$count} = 0;
				__set_tbl_val($dbh, $tbl_head, $count, $hid, dochar(0));
			}
			return $res;
		}
	}
	my $item_id = $head->{$last};
	my $previous = __par_clef($dbh, $tbl_item, $head->{$last},$next);
	if ( ! defined $previous ) {
		debug "[ __list_pop ] there are at least two elements, and I cannot get pre-last one...";
		return undef;
	}
	my $previous_id = $previous->{$tbl_item.'_ID'};
	__set_tbl_val ($dbh, $tbl_head, $last, $hid, do_null_or_char($previous_id));
	$head->{$last} = $previous_id;
	__set_tbl_val ($dbh, $tbl_item, $next, $previous_id, do_null_or_char(-1));
	__set_tbl_val ($dbh, $tbl_item, $up,   $item_id, do_null_or_char(undef)) if $up;
	if ( $count ) {
		$head->{$count}--;
		__set_tbl_val($dbh, $tbl_head, $count, $hid, do_null_or_char($head->{$count}));
	}
	return $item_id;
}

sub __list_pop($$$) {
	my ( $dbh, $list, $h ) = @_;
	my $id = __list_pop_id($dbh, $list, $h);
	if ( $id ) {
		return __par_id($dbh, $list->{'item_table'}, $id);
	} else {
		return undef;
	}
}

sub __list_unshift_new($$$@) {
	my ( $dbh, $list, $h, @infos ) = @_;
	my $id = __new($dbh, $list->{'item_table'}, @infos);
	__list_unshift($dbh, $list, $h, $id);
	return $id;
}

sub __list_unshift($$$$) {
	my ( $dbh, $list, $h, $id ) = @_;
	return undef if (ref $list ne 'HASH');
	my ( $tbl_head, $tbl_item, $next, $first, $last, $up, $count ) = __list_get_info($list);
	my ( $hid, $head ) = __list_get_head($dbh, $list, $h);
	return undef if ! defined $head;

	if ( $head->{$first} eq -1 && $last ) {
		# List is empty
		__set_tbl_val($dbh, $tbl_head, $last,  $hid, do_null_or_char($id));
		$head->{$last} = $id;
	}
	__set_tbl_val($dbh, $tbl_item, $next, $id, do_null_or_char($head->{$first}));
	__set_tbl_val($dbh, $tbl_item, $up  , $id, do_null_or_char($hid)) if $up;
	__set_tbl_val($dbh, $tbl_head, $first, $hid, do_null_or_char($id));
	$head->{$first} = $id;
	if ( $count ) {
		$head->{$count}++;
		__set_tbl_val($dbh, $tbl_head, $count, $hid, do_null_or_char($head->{$count}));
	}
	return __list_get_count($dbh, $list, $head);
}

sub __list_shift($$$) {
	my ( $dbh, $list, $h ) = @_;
	return undef if (ref $list ne 'HASH');
	my ( $tbl_head, $tbl_item, $next, $first, $last, $up, $count ) = __list_get_info($list);
	my ( $hid, $head ) = __list_get_head($dbh, $list, $h);
	return undef if ! defined $head;

	if ( $head->{$first} eq -1 ) {
		# List is empty
		return undef;
	}

	my $id   = $head->{$first};
	my $item = __par_id($dbh, $tbl_item, $id);
	if ( ! defined $item ) {
		debug "[ __list_shift ] did loose its first element. Returning undef...";
		return undef;
	}

	if ( $last && $item->{$next} == -1 ) {
		# This was the last element
		__set_tbl_val($dbh, $tbl_head, $last,  $hid, do_null_or_char(-1));
		$head->{$last} = -1;
	}
	__set_tbl_val($dbh, $tbl_head, $first, $hid, do_null_or_char($item->{$next}));
	$head->{$first} = $item->{$next};
	__set_tbl_val($dbh, $tbl_item, $next,  $id,  do_null_or_char(-1));
	__set_tbl_val($dbh, $tbl_item, $up,    $id,  do_null_or_char(-1)) if $up;
	if ( $count ) {
		$head->{$count}--;
		__set_tbl_val($dbh, $tbl_head, $count, $hid, do_null_or_char($head->{$count}));
	}
	return $item;
}

sub __list_shift_id($$$) {
	my ( $dbh, $list, $h ) = @_;
	my $tmp = __list_shift($dbh, $list, $h);
	if ( $tmp ) {
		return $tmp->{$list->{'item_table'}.'_ID'};
	} else {
		return undef;
	}
}

# Insert an existing element after another element in a list
#
# dbh    : database handler
# list   : list handle
# h      : id of the list's head, or the head itself
# after  : id of the element after we insert
# id     : id of the element to insert
#
# return ???
sub __list_insert_after ($$$$$) {
	my ($dbh, $list, $h, $previous_id, $id ) = @_;
	return undef if (ref $list ne 'HASH');
	if ( $previous_id == -1 ) {
		return __list_unshift($dbh, $list, $h, $id);
	}
	my ( $tbl_head, $tbl_item, $next, $first, $last, $up, $count ) = __list_get_info($list);
	my ( $hid, $head ) = __list_get_head($dbh, $list, $h);

	my $previous = __par_id($dbh, $tbl_item, $previous_id);
	my $item     = __par_id($dbh, $tbl_item, $id);
	if ( $last && $previous->{$next} == -1 ) {
		__set_tbl_val($dbh, $tbl_head, $last, $hid, do_null_or_char($id));
		$head->{$last} = $id;
	}
	__set_tbl_val($dbh, $tbl_item, $up,   $id, dochar($hid)) if $up;
	__set_tbl_val($dbh, $tbl_item, $next, $id, dochar($previous->{$next}));
	__set_tbl_val($dbh, $tbl_item, $next, $previous_id, dochar($id));
	if ( $count ) {
		$head->{$count}++;
		__set_tbl_val($dbh, $tbl_head, $count, $hid, do_null_or_char($head->{$count}));
	}
	return __list_get_count($dbh, $list, $head);
}

# extract an element of the list
#
# dbh    : database handler
# list   : list handle
# h      : id of the list's head, or the head itself
# id     : id of the element
#
# return id of the element
sub __list_detach ($$$$) {
	my ($dbh, $list, $h, $id ) = @_;
	return undef if (ref $list ne 'HASH');
	my ( $tbl_head, $tbl_item, $next, $first, $last, $up, $count ) = __list_get_info($list);
	my ( $hid, $head ) = __list_get_head($dbh, $list, $h);

	my $item = __par_id($dbh, $tbl_item, $id);
	return undef if ! defined $item;

	if ( $up ) {
		__set_tbl_val($dbh, $tbl_item, $up, $id, dochar(-1));
		$item->{$up} = -1;
	}
	if ( $item->{$next} == -1 ) {
		# This is the last item
		if ( $head->{$first} == $id ) {
			# This is the only item
			__set_tbl_val($dbh, $tbl_head, $first, $hid, dochar(-1));
			$head->{$first} = -1;
			if ( $last ) {
				__set_tbl_val($dbh, $tbl_head, $last,  $hid, dochar(-1));
				$head->{$last} = -1;
			}
			if ( $count ) {
				$head->{$count} = 0;
				__set_tbl_val($dbh, $tbl_head, $count, $hid, dochar(0));
			}
			return $item;
		} else {
			# There is a previous one
			my $previous = __par_clef($dbh, $tbl_item, $id, $next);
			__set_tbl_val($dbh, $tbl_head, $last, $hid, dochar($previous->{$tbl_item.'_ID'})) if $last;
			__set_tbl_val($dbh, $tbl_item, $next, $previous->{$tbl_item.'_ID'}, dochar(-1));
			if ( $count ) {
				$head->{$count}--;
				__set_tbl_val($dbh, $tbl_head, $count, $hid, dochar($head->{$count}));
			}
			return $item;
		}
	} elsif ( $head->{$first} == $id ) {
		# This is the first one, and there is at least one more.
		__set_tbl_val($dbh, $tbl_head, $first, $hid, dochar($item->{$next}));
		__set_tbl_val($dbh, $tbl_item, $next,  $id,  dochar(-1));
		$item->{$next} = -1;
		if ( $count ) {
			$head->{$count}--;
			__set_tbl_val($dbh, $tbl_head, $count, $hid, do_null_or_char($head->{$count}));
		}
		return $item;
	} else {
		# Not the first, neither the last
		my $previous = __par_clef($dbh, $tbl_item, $id, $next);
		my $previous_id = $previous->{$tbl_item.'_ID'};
		__set_tbl_val($dbh, $tbl_item, $next, $previous_id, dochar($item->{$next}));
		__set_tbl_val($dbh, $tbl_item, $next, $id, dochar(-1));
		$item->{$next} = -1;
		if ( $count ) {
			$head->{$count}--;
			__set_tbl_val($dbh, $tbl_head, $count, $hid, do_null_or_char($head->{$count}));
		}
		return $item;
	}
}

# Get all elements of the list
#
# dbh    : database handler
# list   : list handle
# h      : id of the list's head, or the head itself
#
# return an ordered list of all elements
sub __list_get_list ($$$) {
	my ( $dbh, $list, $hid ) = @_;
	return undef if (ref $list ne 'HASH');
	if ( $list->{'item_up'} ) {
		return __list_get_list_fullup($dbh, $list, $hid, -1);
	} else {
		return __list_get_list_iter($dbh, $list, $hid, -1);
	}
}

# Get N firsts elements of the list
#
# dbh    : database handler
# list   : list handle
# h      : id of the list's head, or the head itself
# limit  : number of element
#
# return an ordered list of N firsts elements
sub __list_get_list_n ($$$$) {
	my ($dbh, $list, $h, $limit ) = @_;
	return undef if (ref $list ne 'HASH');
	if ( $list->{'item_up'} ) {
		return __list_get_list_fullup($dbh, $list, $h, $limit);
	} else {
		return __list_get_list_iter($dbh, $list, $h, $limit);
	}
}

sub __list_get_list_fullup($$$;$) {
	my ($dbh, $list, $h, $limit ) = @_;
	my ( $tbl_head, $tbl_item, $next, $first, $last, $up, $count ) = __list_get_info($list);
	my ( $hid, $head ) = __list_get_head($dbh, $list, $h);
	return undef if ! defined $head;

	my $res = [ ];
	my $items = [ __par_clef($dbh, $tbl_item, $hid, $up) ];
	my $item_h = FDN::Sql::Basic::__make_h( $items, $tbl_item.'_ID');
	my $id = $head->{$first};
	while ( $id > 0 && $limit && exists $item_h->{$id} ) {
		my $tmp = $item_h->{$id};
		delete $item_h->{$id};
		push @$res, $tmp;
		$id = $tmp->{$next};
		$limit--;
	}
	return $res;
}

sub __list_get_list_iter($$$;$) {
	my ( $dbh, $list, $h, $limit ) = @_;
	$limit = -1 if ! defined $limit;
	my ( $tbl_head, $tbl_item, $next, $first, $last, $up, $count ) = __list_get_info($list);
	my ( $hid, $head ) = __list_get_head($dbh, $list, $h);
	return undef if ! defined $head;
	my $id   = $head->{$first};
	return __list_get_next_n($dbh, $list, $h, $id, $limit, 0);
}

sub __list_get_relative_n($$$$$) {
	my ( $dbh, $list, $h, $id, $limit ) = @_;

	if ( $limit > 0 ) {
		return __list_get_next_n($dbh, $list, $h, $id, $limit, 0 );
	} elsif ( $limit < 0 ) {
		return __list_get_previous_n($dbh, $list, $h, $id, -$limit );
	} else {
		return [];
	}
}

sub __list_get_next_n($$$$$;$) {
	my ( $dbh, $list, $h, $id, $limit, $exclusive ) = @_;

	my ( $tbl_head, $tbl_item, $next, $first, $last, $up, $count ) = __list_get_info($list);

	my $tmp;
	my $res = [];
	my $hash = {};
	if ( $exclusive ) {
		$tmp = __par_id($dbh, $tbl_item, $id);
		$hash->{$id} = $tmp;
		$id = $tmp->{$next};
	}

	while ( $id > 0 && $limit && ! exists $hash->{$id} ) {
		$tmp = __par_id($dbh, $tbl_item, $id);
		$hash->{$id} = $tmp;
		push @$res, $tmp;
		$id = $tmp->{$next};
		$limit--;
	}
	return $res;
	
}

sub __list_get_previous_n($$$$$) {
	my ( $dbh, $list, $h, $id, $limit ) = @_;

	my ( $tbl_head, $tbl_item, $next, $first, $last, $up, $count ) = __list_get_info($list);
	my $res = [];
	my $hash = {};
	my $previous = __par_clef($dbh, $tbl_item, $id, $next);
	my $previous_id = $previous->{$tbl_item.'_ID'};
	while ( $previous && $limit && ! exists $hash->{$previous_id} ) {
		$hash->{$previous_id} = $previous;
		push @$res, $previous;
		$limit--;
		$previous = __par_clef($dbh, $tbl_item, $previous_id, $next) if $limit;
		$previous_id = $previous->{$tbl_item.'_ID'};
	}
	$res = [ reverse @$res ];
	return $res;

}

# Get number of element im a list
#
# dbh    : database handler
# list   : list handle
# h      : id of the list's head, or the head itself
#
# return the number of element 
sub __list_get_count ($$$) {
	my ($dbh, $list, $h ) = @_;
	return undef if (ref $list ne 'HASH');
	my ( $tbl_head, $tbl_item, $next, $first, $last, $up, $count ) = __list_get_info($list);
	my ( $hid, $head );
	if ( $count ) {
		( $hid, $head ) = __list_get_head($dbh, $list, $h);
		return $head->{$count};
	} elsif ( $up ) {
		$hid = __list_get_head_id($dbh, $list, $h);

		my $name = $tbl_item.'.'.$tbl_item.'_ID';
		my $tmp = get_data_reuse($dbh, [ $name ], [ $tbl_item.'.'.$up.'='.$hid ], [ "count($name)" ] );
		if ( ! $tmp->{'ok'} ) {
			return -1;
		}
		my $res = $tmp->{'rows'}->[0]->{"count($name)"};
		return $res;
	} else {
		return -1;
	}
}

# Get IDs of all elements of the list
#
# dbh    : database handler
# list   : list handle
# hid    : id of the list's head
#
# return an ordered list of all Ids elements
sub __list_get_list_ids ($$$) {
	my ($dbh, $list, $hid ) = @_;
}

# Get IDs of N firsts elements of the list
#
# dbh    : database handler
# list   : list handle
# hid    : id of the list's head
# limit  : number of element
#
# return an ordered list of N Ids firsts elements
sub __list_get_list_ids_n ($$$$) {
	my ($dbh, $list, $hid, $limit ) = @_;
}

# Get IDs of N IDs elements after another of the list
#
# dbh    : database handler
# list   : list handle
# hid    : id of the list's head
# from   : id of last element before return
# limit  : number of element
#
# return an ordered list of N Ids firsts elements
sub __list_get_list_ids_n_from ($$$$$) {
	my ($dbh, $list, $hid, $from, $limit ) = @_;
}


1;
