package FDN::Sql::New;

use strict;
use Exporter;
our @ISA = ( 'Exporter' );
our @EXPORT = qw (
	__renseigne
	__modifie
	__set_form_config_file
	__form
	);
our @EXPORT_OK = qw (
);
our %EXPORT_TAGS = qw (
);

use FDN::Common::Logger;
use FDN::Common::Config;
use FDN::Web::Forms qw( :dynamics );
use FDN::Sql::Basic;

our $debug = 0;
sub __debug($) {
	return if ! $debug;
	debug $_[0];
}


sub __base_tbl($) {
	my ( $tbl ) = @_;
	if ( ! defined $tbl->{'NAME'} ) {
		error "Poorly defined tbl, missing NAME";
		die "Horribly!";
	}
	if ( ! defined $tbl->{'DBH'} ) {
		error "Poorly defined tbl, missing DBH";
		die "Horribly!";
	}
	if ( ! defined $tbl->{'EQUIV'} ) {
		error "Poorly defined tbl, missing EQUIV";
		die "Horribly!";
	}
	__debug "Base tbl for ".$tbl->{'NAME'}." seems ok";
}

sub __set_id_fct($) {
	my ( $tbl ) = @_;
	if ( ! defined $tbl->{'PAR_ID'} ) {
		__base_tbl($tbl);
		$tbl->{'PAR_ID'} = sub { my($id)=@_; return __par_id($tbl->{'DBH'},$tbl->{'NAME'},$id); };
		__debug "PAR_ID have been created for ".$tbl->{'NAME'};
	}
	return;
}

sub __set_modif_hash($) {
	my ( $tbl, $hfld ) = @_;
	if ( ! defined $tbl->{'MODIF'} ) {
		__base_tbl($tbl);
		my $eq = $tbl->{'EQUIV'};
		my $mod = {};
		for my $hfld ( keys %$eq ) {
			my $inf = $mod->{$hfld};
			my $sfld = $tbl->{'EQUIV'}->{$hfld};
			my $type = 'char';
			if ( defined $tbl->{'TYPE'} && defined $tbl->{'TYPE'}->{$sfld} ) {
				$type = $tbl->{'TYPE'}->{$sfld};
			}
			if ( $sfld !~ /_/ ) {
				$sfld = $tbl->{'NAME'}.'_'.$sfld;
			}
			$inf->[0] = $sfld;
			if ( $type eq 'date' ) {
				$inf->[1] = sub { my ( $id, $val ) = @_; return __set_tbl_val($tbl->{'DBH'},$tbl->{'NAME'},$inf->[0],$id,dodate($val)); };
			} else {
				$inf->[1] = sub { my ( $id, $val ) = @_; return __set_tbl_val($tbl->{'DBH'},$tbl->{'NAME'},$inf->[0],$id,dochar($val)); };
			}
			$mod->{$hfld} = $inf;
		}
		$tbl->{'MODIF'} = $mod;
		__debug "MODIF hash have been created for ".$tbl->{'NAME'};
	}
	return;
}

sub __set_equiv_hash($) {
	my ( $tbl ) = @_;
	if ( ! defined $tbl->{'EQUIV'} ) {
		if ( ! defined $tbl->{'MODIF'} ) {
			error "There is no EQUIV and no MODIF. Poorly defined tbl.";
			die "Horribly!";
		}
		my $eq = {};
		my $mod = $tbl->{'MODIF'};
		for my $k ( keys %$mod ) {
			$eq->{$k} = $mod->{$k}->[0];
		}
		$tbl->{'EQUIV'} = $eq;
		__debug "EQUIV hash have been created for ".$tbl->{'NAME'};
	}
	return;
}

sub __renseigne($$$) {
	my ( $tbl, $vars, $id ) = @_;
	if ( ! defined $id ) {
		return;
	}
	__set_id_fct($tbl);
	my $fct = $tbl->{'PAR_ID'};
	my $infos = &$fct($id);
	if ( ! defined $infos ) {
		return;
	}
	__set_equiv_hash($tbl);
	my $eq = $tbl->{'EQUIV'};
	for my $k ( keys %$eq ) {
		my $field = $eq->{$k};
		if ( $field !~ /_/ ) {
			$field = $tbl->{'NAME'}.'_'.$field;
		}
		$vars->{$k} = $infos->{$field};
		__debug "Field $k from ".$field." is ".$infos->{$field};
	}
	return;
}

sub __create_auto($$) {
	my ( $tbl, $vars ) = @_;
	my @params;
	__base_tbl($tbl);
	my $mod = $tbl->{'EQUIV'};
	for my $k ( keys %$mod ) {
		my $type = 'char';
		if ( defined $tbl->{'TYPE'} && defined $tbl->{'TYPE'}->{$mod->{$k}} ) {
			$type = $tbl->{'TYPE'}->{$mod->{$k}};
		}
		if ( $type eq 'date' ) {
			push @params, $mod->{$k}.'='.dodate($vars->{$k});
		} else {
			push @params, $mod->{$k}.'='.dochar($vars->{$k});
		}
	}
	my $id = __new($tbl->{'DBH'}, $tbl->{'NAME'}, @params);
	__debug "New ".$tbl->{'NAME'}." have been created automaticaly, id $id";
	return $id;
}

sub __create($$) {
	my ( $tbl, $vars ) = @_;
	# Build parameters list from NEW
	my @params;
	my $new = $tbl->{'NEW'};
	for my $k ( keys %$new ) {
		if ( defined $new->{$k} and ref($new->{$k}) eq '' ) {
			if ( defined $vars->{$k} ) {
				$params[$new->{$k}] = $vars->{$k};
			} else {
				$params[$new->{$k}] = $k;
			}
		}
	}

	# Call the NEW fct (new_XXX)
	my $fct = $tbl->{'CREATE'};
	my $id = &$fct(@params);

	# Call the direct update functions
	for my $k ( keys %$new ) {
		next if ! defined $vars->{$k};
		if ( ref($new->{$k}) eq 'CODE' ) {
			my $fct = $new->{$k};
			&$fct($id,$vars->{$k});
		}
	}
	
	# Call the reverse update functions
	for my $k ( keys %$new ) {
		next if ! defined $vars->{$k};
		if ( ref($new->{$k}) eq 'ARRAY' ) {
			my $fct = $new->{$k}->[0];
			&$fct($vars->{$k},$id);
		}
	}
	__debug "New ".$tbl->{'NAME'}." have been created from functions, id $id";
	return $id;
}

sub __modifie($$$) {
	my ( $tbl, $vars, $id ) = @_;
	goto CREE if ( ! defined $id );
	goto CREE if ( $id < 0 );
	__set_id_fct($tbl);
	my $infos = &{$tbl->{'PAR_ID'}}($id);
	goto CREE if ( ! defined $infos );
	__set_modif_hash($tbl);
	my $mod = $tbl->{'MODIF'};
	__debug "Il faut modifier l'objet.";
	for my $k ( keys %$mod ) {
		my $field = $mod->{$k}->[0];
		my $fct = $mod->{$k}->[1];
		next if ! defined $vars->{$k};
		$infos->{$field} = 'undef' if ! defined $infos->{$field};
		if ( $vars->{$k} ne $infos->{$field} ) {
			__debug "Le champ $k ne correspond pas avec $field, il faut mettre a jour.";
			&$fct($id,$vars->{$k});
		}
	}
	return $id;
CREE:
	__debug "Il faut creer l'objet.";
	my @params;
	if ( ! defined $tbl->{'CREATE'} ) {
		$id = __create_auto($tbl, $vars);
	} else {
		$id = __create($tbl, $vars);
	}
	return $id;
}

my $form_file;
sub __set_form_config_file($) {
	my ( $file ) = @_;
	$form_file = $file;
}

sub __add_field_from_config($$$$;$) {
        my ( $form, $cfg_file, $block, $champ, $placement ) = @_;
        my $cfg = Load_Config($cfg_file);
        my $fld = form_new_field($cfg,$champ);
        my $blk = $form->{'blocksbyname'}->{$block};
        form_add_field($blk,$fld,$placement);
}

sub __form($$) {
	my ( $tbl, $vars ) = @_;
	if ( ! defined $form_file ) {
		error "If you use __form in this module, you need to use previously __set_form_config_file";
		$form_file = '/dev/null';
	}
	my $form = form_read($form_file,$tbl->{FORM});
	for my $h ( @{$tbl->{'HIDDEN'}} ) {
		my ( $input, $block, $field ) = @$h;
		if ( defined $vars->{$input} && $vars->{$input} ) {
			__add_field_from_config($form,$form_file, $block, $field );
		}
	}
	return $form if ! defined $vars->{'id'};

	__set_id_fct($tbl);
	my $fct = $tbl->{'PAR_ID'};
	my $truc = &$fct($vars->{'id'});
	return $form if ! defined $truc;
	
	my $html = $tbl->{'HTMLIDS'};
	my $fromid = $vars->{'id'};
	for my $k ( keys %{$html} ) {	
		next if ( ! defined $truc->{$k} );
		next if ( ! $truc->{$k} > 0 );
		my $toid = $truc->{$k};
		my $fld = $form->{'byname'}->{$html->{$k}};
		my $tmp = $fld->{'content'};
		$tmp =~ s/FROMID/$fromid/g;
		$tmp =~ s/TOID/$toid/g;
		form_set_field_value($fld,$tmp);
	}
	return $form;
}


1;


