# This file is part of the "FDN tools" project.
# You can use and redistribute it under the terms of the
# GNU General Public License.
# (c) French Data Network 1997-2005
# Authors and contributors:
# - Benjamin Bayart
# - Alexis Sukrieh
package FDN::Sql::Plutar::Basic;

use strict;
use Exporter ;
our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
	plutar_select_db
	plutar_close_db

	lock_plutar
	lock_propplutar
	unlock_plutar
	unlock_propplutar
	
	set_plutar_properties
	get_plutar_properties
	del_plutar_properties
	update_plutar_properties
	
	new_plutar
	del_plutar
	plutar_par_id
	plutar_par_propplutar
	plutar_par_action_propplutar
	first_plutar
	next_plutar
	delta_date
	plutar_etat_par_id
	update_task_field
	update_task_dt
	
);
our @EXPORT_OK = ();

use FDN::Common::Logger;
use FDN::Common::Constantes;
use FDN::Common::Modules;
use FDN::Sql::Basic;
use FDN::Sql::DB_Access; # For lock_tables and sql_transaction
use FDN::Sql::DBC;
use FDN::Sql::Plutar::Constantes;
use POSIX qw(strftime);
use Data::Dumper;

my $pdb = undef;
my $dbh = undef;
our $debug = 0;

sub __debug($) { # Test
	return if ( ! $debug );
	debug $_[0];
}

sub plutar_select_db ($) { # Test
	my ( $db ) = @_;
	if ( defined $dbh ) {
		info "You are already connected to a Plutar database, and did ask to switch to $db";
	}
	$dbh = do_connect($db);
	return $dbh;
}

sub plutar_close_db() {
	if ( ! defined $dbh ) {
		error "You did ask to close Plutar databse, while none is selected...";
		return;
	}
	do_disconnect($dbh);
}

# Table PLUTAR

sub new_plutar ($$) { # Test
	my ($action, $dtdo) = @_;
	return __new ( $dbh, 'PLUTAR',
		'PLUTAR_ACTION='.dochar($action),
		'PLUTAR_DTDO='.dodate($dtdo),
		'PLUTAR_DTWAIT='.dodate($dtdo),
		'PLUTAR_ETAT='.dochar('waiting'),
		'PLUTAR_DTMOD=now()');
}

sub del_plutar_properties($) {
	my ( $id ) = @_;
	return __del_properties($dbh, 'PLUTAR', $id);
}

sub del_plutar($) {
	my ( $id ) = @_;
	return __del($dbh, 'PLUTAR', $id);
}

sub plutar_par_id($) { # Test
	my ( $id ) = @_;
	return __par_id($dbh,'PLUTAR',$id);
}

sub plutar_par_propplutar ($$) { # Test
	my ($nom, $val) = @_;
	return __par_champ_ext($dbh,'PLUTAR',
			[ [ 'PROPPLUTAR_NOM', '=', dochar($nom) ],
			  [ 'PROPPLUTAR_VAL', '=', dochar($val) ],
			  [ 'PROPPLUTAR.PLUTAR_ID', '=', 'PLUTAR.PLUTAR_ID' ]],
			  { 'misc' => 'distinct' });
}

sub plutar_par_action_propplutar ($$$) { # Test
	my ($action, $nom, $val) = @_;
	return __par_champ_ext($dbh,'PLUTAR',
			[ [ 'PLUTAR_ACTION', '=', dochar($action) ],
			  [ 'PROPPLUTAR_NOM', '=', dochar($nom) ],
			  [ 'PROPPLUTAR_VAL', '=', dochar($val) ],
			  [ 'PROPPLUTAR.PLUTAR_ID', '=', 'PLUTAR.PLUTAR_ID' ]],
			  { 'misc' => 'distinct' });
}

sub plutar_etat_par_id ($) {
	my ($pid) = @_;
	return __champ_par_id($dbh, 'PLUTAR', 'PLUTAR_ETAT', $pid);
}

sub first_plutar() { # Test
	return __par_champ_ext ( $dbh, 'PLUTAR',
		[ [ 'PLUTAR_ETAT', '=',  dochar('waiting') ],
		  [ 'PLUTAR_DTDO', '<=', 'now()'  ] ],
		{ 'order' => 'orderup(PLUTAR_DTDO)', 'range' => '0-0' } );
}

sub next_plutar() {
	return __par_champ_ext ( $dbh, 'PLUTAR',
		[ [ 'PLUTAR_ETAT', '=',  dochar('waiting') ] ],
		{ 'order' => 'orderup(PLUTAR_DTDO)', 'range'=>'0-0' } );
}

sub delta_date($) {
	my ( $date ) = @_;
	my $res = sql_transaction($dbh,'select',"select TIMESTAMPDIFF(SECOND,now(),'$date') as res");
	return FDN::Sql::Basic::__return_one_col($res,'res');
}


sub update_task_field ($$$) { # Test
	my ($plutar_id, $champ, $valeur) = @_;
	my $champ_name = 'PLUTAR_'.uc($champ);
	
	return __set_tbl_val (
		$dbh		, 
		'PLUTAR'	,
		$champ_name	,
		$plutar_id	,
		dochar($valeur)
	);
}

sub update_task_dt ($$$) { # Test
	my ($plutar_id, $champ, $valeur) = @_;
	my $champ_name = 'PLUTAR_'.uc($champ);
	
	return __set_tbl_val (
		$dbh		, 
		'PLUTAR'	,
		$champ_name	,
		$plutar_id	,
		$valeur
	);
}

sub lock_plutar () { # Test
	return lock_soft($dbh,'plutar');
}

sub unlock_plutar () { # Test
	return unlock_soft($dbh,'plutar');
}

# Table PROPPLUTAR

sub get_plutar_properties ($) { # Test
	my ($id) = @_;
	return __get_properties($dbh,'PLUTAR',$id);
}

sub set_plutar_properties($$) { # Test
	my ( $id, $args ) = @_;
	return __set_properties($dbh,'PLUTAR',$id,$args);
}

sub update_plutar_properties($$) { # Test
	my ( $id, $args ) = @_;
	return __update_properties($dbh,'PLUTAR',$id,$args);
}

1 ;
