# This file is part of the "FDN tools" project.
# You can use and redistribute it under the terms of the
# GNU General Public License.
# (c) French Data Network 1997-2005
# Authors and contributors:
# - Alexis Sukrieh
# - Benjamin Bayart
package FDN::Sql::Plutar::Run;

use strict;
use Exporter ;
our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
	plutar_select_db
	plutar_close_db
	plutar_select_config
	
	first_plutar
	plutar_run_task
	plutar_after_run
	plutar_send_notify
	purge_plutar
	
	plutar_cancel_task
	plutar_retry_task
	plutar_waiting_task
	plutar_runing_task
	plutar_done_task
	plutar_fail_task
);
our @EXPORT_OK = qw (
	__plutar_update_task
);

use FDN::Common::Logger;
use FDN::Common::Constantes;
use FDN::Common::Modules;
use FDN::Common::Config;
use FDN::Sql::Basic;
use FDN::Sql::Plutar::Constantes;
use FDN::Sql::Plutar::Basic;
use POSIX qw(strftime);
use Mail::Sendmail;

our $debug = 0;

sub __debug($) {
	return if ( ! $debug );
	debug $_[0];
}

my $config = 'plutar.cfg';
sub plutar_select_config($) {
	$config = $_[0];
	my $tmp = Load_Config($config);
	if ( ! defined $tmp ) {
		return FALSE;
	}
	return TRUE;
}

# ----------------------------------------------------------------------
# nom 		: plutar_run_task 
# description 	: effectue une tache de plutar
# parametres 	: args = les arguments de la tache dans plutar
#                        sous forem de properties
#                 name = le nom de la fonction a executer
# retours 	:  
# ----------------------------------------------------------------------
sub plutar_run_task ($$) {
	my ( $args, $type ) = @_;
	unless( defined($args) and ref($args) eq 'HASH' and
		defined($type) and length($type) ){
		error("Wrong parameters");
		return FALSE;
	}
	my $name = Config_Key($config,$type,'fonction');
	unless( defined($name) and length($name) ) {
		error "Action not defined for task type $type";
		return PLUTAR_FAILED;
	}
	my $real = get_foreign_function($name);
	if ( ref($real) ne 'CODE' ) {
		error "Function $name does not exist.";
		return PLUTAR_FAILED;
	}
	my $res;
	eval { $res = &$real($args); };
	if ( !defined $res )  {
		error "Action '$type' failed with error: $@. Will retry.";
		return PLUTAR_RETRY;
	}
	return $res;
}

sub plutar_get_notify() {
	my $name = Config_Key($config,'common','notify');
	unless ( defined($name) and length($name) ) {
		return '';
	}
	my $real = get_foreign_function($name);
	if ( ref($real) ne 'CODE' ) {
		error "Function $name does not exist.";
		return undef;
	}
	return &$real();
}

# ----------------------------------------------------------------------
# nom           : __update_plutar_task
# description   : modifie la tache en fonction du code retour renvoy par 
#                 l'appel perl de la tache 
# parametres    : plutar_id
#                 status_task
#                 status_time
#                 time
# retours       : 0 ou 1
# ----------------------------------------------------------------------
sub __plutar_update_task ($$$$) {
	my ( $pid, $status, $timer, $time) = @_;

	unless (defined($pid) && length($pid) &&
	        defined($status) && length($status) &&
		defined($timer) && length($timer)
		) {
		error("Wrong parameters");
		return FALSE;
	}
	if ( ! defined $time ) {
		$time = 0;
	}
	if ( ! update_task_field($pid,'ETAT',$status) ) {
		error "Failed to update task. Unlocking and getting away.";
		return FALSE;
	}
	$time += 0;
	$time = "DATE_ADD(now(), INTERVAL $time SECOND)";
	if ( ! update_task_dt($pid,$timer,$time) ) {
		error "Failed to update task. Unlocking and getting away.";
		return FALSE;
	}
	return TRUE;
}


# ----------------------------------------------------------------------
# nom           : plutar_cancel_task 
# description   : passe une tache de plutar a l'etat cancel
# parametres    : plutar_id
# retours       : 0 ou 1
# ----------------------------------------------------------------------
sub plutar_cancel_task ($) {
	my ( $pid ) = @_;
	return __plutar_update_task ( $pid, 'canceled', 'CANCELED',0);

}

# ----------------------------------------------------------------------
# nom           : plutar_fail_task 
# description   : passe une tache de plutar a l'etat failed
# parametres    : plutar_id
# retours       : 0 ou 1
# ----------------------------------------------------------------------
sub plutar_fail_task ($) {
	my ( $pid ) = @_;
	unless (defined($pid) && length($pid)) {
		error("Wrong parameters");
		return FALSE;
	}
	return __plutar_update_task ( $pid, 'failed', 'DTFAILED', 0);
}

# ----------------------------------------------------------------------
# nom           : plutar_done_task 
# description   : passe une tache de plutar a l'etat done
# parametres    : plutar_id
# retours       : 0 ou 1
# ----------------------------------------------------------------------
sub plutar_done_task ($) {
	my ( $pid ) = @_;
	unless (defined($pid) && length($pid)) {
		error("Wrong parameters.");
		return FALSE;
	}
	return __plutar_update_task ( $pid, 'done', 'DTDONE', 0);
}

# ----------------------------------------------------------------------
# nom           : plutar_waiting_task 
# description   : passe une tache de plutar a l'etat waiting
# parametres    : plutar_id
# retours       : 0 ou 1
# ----------------------------------------------------------------------
sub plutar_runing_task ($$) {
	my ( $pid, $run ) = @_;
	unless (defined($pid) && length($pid)) {
		error("Wrong parameters");
		return FALSE;
	}
	update_task_field($pid,'NBRUN',$run);
	return __plutar_update_task ( $pid, 'runing', 'DTRUN', 0);
}

# ----------------------------------------------------------------------
# nom           : plutar_waiting_task 
# description   : passe une tache de plutar a l'etat waiting
# parametres    : plutar_id
# retours       : 0 ou 1
# ----------------------------------------------------------------------
sub plutar_waiting_task ($) {
	my ( $pid ) = @_;
	unless (defined($pid) && length($pid)) {
		error("Wrong parameters");
		return FALSE;
	}
	return __plutar_update_task ( $pid, 'waiting', 'DTWAIT', 0 );
}

# ----------------------------------------------------------------------
# nom           : plutar_retry_task 
# description   : passe une tache de plutar a l'etat retry
# parametres    : plutar_id
#                 retry_delay = delai entre deux essais
# retours       : 0 ou 1
# ----------------------------------------------------------------------
sub plutar_retry_task ($$) {
	my ( $pid, $delay ) = @_;
	unless (defined($pid)   && length($pid) &&
	        defined($delay) && length($delay)
	        ) {
		error("Wrong parameters");
		return FALSE;
	}
	return __plutar_update_task ( $pid, 'waiting', 'DTDO', $delay);
}

sub plutar_after_run($$$) {
	my ( $pid, $ret, $act ) = @_;
	my $task = plutar_par_id($pid);
	my $run  = $task->{'PLUTAR_NBRUN'};
	my $maxrun = Config_Key($config,$act,'maxretry');
	my $delay  = Config_Key($config,$act,'retrydelay');

	if ( $task->{'PLUTAR_ETAT'} ne 'runing' ) {
		if ( $task->{'PLUTAR_ETAT'} eq 'canceled' ) {
			return "Plutar task $pid have been canceled while it ".
				"was runing. I decide to drop the result and ".
				"leave it known as canceled";
		}
		error("The plutar task $pid have been modified while runing. ".
			"I did put it in 'runing' state. And now, after working, ".
			"I find it is in '$task->{'PLUTAR_ETAT'}' state. I would ".
			"understand 'canceled', but anything else is probably ".
			"wrong.");
		return "Plutar task $pid have been modified. ALARAM !";
	}
	
	if ($ret == PLUTAR_SUCCESS) {
		plutar_done_task($pid);
		return "Execution of plutar task $pid ok";
	}
	if ($ret == PLUTAR_FAILED) {
		plutar_fail_task($pid);
		return "Execution of plutar task $pid failed";
	}
	if ($ret == PLUTAR_RETRY) {
		# si le nombre d'essais max autorise un nouvel essai
		if ($run < $maxrun) {
			plutar_retry_task($pid,$delay);
			return "Execution of plutar task $pid retry within $delay seconds.";
		} else {
			plutar_fail_task($pid);
			return  "No more trial for plutar task $pid (already $maxrun), ".
				"thus task failed.";
		}
	}
	if ($ret == FALSE) {
		return  "Something wrong. That is probably a bug of yours, ".
			"try to guess from previous errors (troubles connecting ".
			"to database or such)."
	}
	return "The return code '$ret' is not valid. Expect a serious bug in ".
		"Plutar, you should check for the code corresponding to that ".
		"action in your module!";
}

sub __need_to_notify($$) {
	my ( $ret, $action ) = @_;
	my $etat;
	$etat = 'success' if $ret == PLUTAR_SUCCESS;
	$etat = 'retry'   if $ret == PLUTAR_RETRY;
	$etat = 'failed'  if $ret == PLUTAR_FAILED;
	my $key = 'notify_'.$etat;
	my $to = Config_Key($config,$action,$key);
	if ( ! defined $to ) {
		__debug "There is no need for a notification for result $etat of $action.";
		return FALSE;
	}
	return $to;
}

sub __make_mail_subject($$) {
	my ( $ret, $task ) = @_;
	my $act = $task->{'PLUTAR_ACTION'};
	my $nbrun = $task->{'PLUTAR_NBRUN'};
	my $maxrun = Config_Key($config,$act,'maxretry');
	my $etat;
	$etat = 'INVALID';
	$etat = 'Retry'   if $ret == PLUTAR_RETRY;
	$etat = 'Failed'  if $ret == PLUTAR_FAILED;
	$etat = 'Success' if $ret == PLUTAR_SUCCESS;
	return "[ Plutar $etat $act] Trying $nbrun/$maxrun";
}

sub __make_mail_text($$) {
	my ( $ret, $task ) = @_;
	my $text;
	$text .= "\t- Type of the Plutar task   : ".$task->{'PLUTAR_ACTION'}."\n";
	$text .= "\t- Id of the Plutar task     : ".$task->{'PLUTAR_ID'}."\n";
	$text .= "\t- Current state of the task : ".$task->{'PLUTAR_ETAT'}."\n";
	$text .= "\t- Date of the last run      : ".$task->{'PLUTAR_DTMOD'}."\n";
	if ( $ret == PLUTAR_RETRY ) {
		$text .= "\t- Date of the next run      : ".$task->{'PLUTAR_DTDO'}."\n";
	}
	$text .= "-" x 79 . "\n\n";
}

sub plutar_send_notify($$$) {
	my ( $pid, $ret, $action ) = @_;

	# Should we notify?
	my $from = Config_Key($config,'common','from');
	if ( !defined $from || ! $from ) {
		__debug "No 'from' in the 'common' section, so no notification.";
		return FALSE;
	}
	my $to = __need_to_notify($ret,$action);
	if ( ! $to ) {
		return FALSE;
	}
	
	# ventuellement, on rcupre les dtails de la tche dans la variable globale
	# NOTIFY_STR
	my $notify = plutar_get_notify();
	if (defined $notify ) {
		__debug "Message found for notify, we use it.";
	}

	# on met  jour notre copie mmoire de la tache
	my $task = plutar_par_id($pid);
	if (! defined $task) {
		error "Impossible de ractualiser la copie mmoire de la tache en cours !";
		return FALSE;
	}

	# Construction du mail
	my $sujet = __make_mail_subject($ret,$task);
	my $text  = __make_mail_text($ret,$task);
	if ( defined $notify ) {
		$text .= "Additional information provided by the task itself:\n";
		$text .= $notify;
	} else {
		$text .= "The task did not provide any additional information.\n";
	}
	
	my $mail_ok = sendmail(
			'To' => $to,
			'From' => $from,
			'Message' => $text,
			'Subject' => $sujet
			);
	if ($mail_ok) {
		info "Le mail a t correctement envoy: ".$Mail::Sendmail::log;
		return TRUE;
	} else {
		error "Probleme lors de l'envoi du mail: ".$Mail::Sendmail::error;
		info "Les autres informations fournies par Mail::Sendmail: ".$Mail::Sendmail::log;
		return FALSE;
	}
}

sub purge_plutar($);
sub purge_plutar($) { # Test
	my ( $tid ) = @_;
	# On supprime ses proprietes
	del_plutar_properties($tid);
	# On degage le plutar, maintenant
	del_plutar($tid);
}

1 ;
