package FDN::Sql::Tags;

use strict;
use Exporter;
use FDN::Sql::Basic qw (
	__new
	__set_relation
);
use FDN::Sql::DB_Access;
use FDN::Common::Logger;

our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
		__get_tag_count_n
		__get_subtag_count_n
		__get_tagged_n
		__set_tags_n
		__update_tags_n

		__get_tag_count
		__get_subtag_count
		__get_tagged
		__set_tags
		__update_tags

		__del_tags
                );
our @EXPORT_OK = qw (
);

our $debug=0;
sub __debug {
	debug @_ if $debug;
}

### sub update_tags_xxx ($$$) {
### 	my ( $Xid, $tags, $owner ) = @_;
### 	return __update_tags_n($dbh, 'XXX', $Xid, $tags, $owner);
### }
### 
### sub tag_par_xxx($) {
### 	my ( $did ) = @_;
### 	return __get_related_a($dbh, 'TAG', 'XXX', $did);
### }
### 
### sub get_tagxxx_count(@) {
### 	return __get_tag_count($dbh,'XXX',@_);
### }
### 
### sub get_tagged_xxxs(@) {
### 	return __get_tagged($dbh,'XXX',@_);
### }

sub __new_tag($$;$) {
	my ( $dbh, $name, $owner ) = @_;
	my @vals = ('TAG_NAME='.dochar($name));
	if ( $owner ) {
		push @vals, 'TAG_OWNER='.dochar($owner);
	}
	return __new($dbh,'TAG',@vals);
}

sub __id_par_clefs( $$$ ) {
	my ( $dbh, $what, $queries ) = @_;

	return undef if ( ! defined $queries or ref($queries) ne 'ARRAY' );
	my $sql = [];
	for my $q ( @$queries ) {
		my ( $id, $clef ) = @$q;
		push @$sql, $what.'.'.$clef."=".$id;
	}
	my $tmp = get_data_reuse ( $dbh, [ $what.'_ID' ], $sql );
	return FDN::Sql::Basic::__return_all_or_one_col($tmp,$what.'_ID');
}



# Penser à supprimer FDN::Sql::DB_Access
sub __resolve_tag_names($$;$$) {
	my ( $dbh, $tags, $owner, $create ) = @_;
	my $tids = [];
	for my $tag ( @$tags ) {
		my $q = [ [ dochar($tag), 'TAG_NAME' ] ];
		push @$q, [ dochar($owner), 'TAG_OWNER' ] if ( defined $owner );
		my $tid = __id_par_clefs($dbh, 'TAG', $q);
		if ( ! defined $tid ) {
			if ( $create ) {
				__debug "Need to create new tag $tag for owner $owner";
				$tid = __new_tag($dbh,$tag,$owner);
			} else {
				__debug "Failed to resolve unknown tag $tag into an id, dropping.";
				next;
			}
		}
		push @$tids, $tid;
	}
	return $tids;
}

sub __get_subtag_count_n($$$;$$$) {
	my ( $dbh, $what, $tags, $owner, $elems, $conds ) = @_;
	my $tids = __resolve_tag_names($dbh, $tags, $owner);
	return __get_subtag_count($dbh, $what, $tids, $elems, $conds);
}

sub __get_subtag_count($$$;$$) {
	my ( $dbh, $what, $tags, $elems, $conds ) = @_;
	$conds = [] if ! defined $conds;
	$elems = [] if ! defined $elems;
	unshift @$elems, $what;
	unshift @$conds, "TAG$what.TAG_ID=TAG.TAG_ID";

	my $tables = ["TAG$what",'TAG'];
	my $i = 0;
	for my $tid ( @$tags ) {
		push @$conds, "T$i.${what}_ID = TAG${what}.${what}_ID", "T$i.TAG_ID = ".dochar($tid), "TAG.TAG_ID != ".dochar($tid);
		push @$tables, "TAG$what as T$i";
		$i++;
	}
	my $tmp = get_data($dbh,['TAG.TAG_NAME','TAG.TAG_ID'], $conds, ['count(TAG.TAG_ID)', "group(TAG$what.TAG_ID)"], $tables );
	return FDN::Sql::Basic::__return_all_or_one($tmp);
}

sub __get_tag_infos($$$$$$) {
	my ( $dbh, $what, $tags, $elems, $conds, $options ) = @_;
	my $tables = [];
	my $i = 0;
	for my $tid ( @$tags ) {
		push @$conds, "T$i.${what}_ID = $what.${what}_ID", "T$i.TAG_ID = ".dochar($tid);
		push @$tables, "TAG$what as T$i";
		$i++;
	}
	my $tmp = get_data($dbh, $elems, $conds, $options, $tables );
}

sub __get_tag_count_n($$$;$$) {
	my ( $dbh, $what, $tags, $owner, $conds ) = @_;
	my $tids = __resolve_tag_names($dbh, $tags, $owner);
	return __get_tag_count($dbh, $what, $tags, $conds);
}

sub __get_tag_count($$$;$) {
	my ( $dbh, $what, $tags, $conds ) = @_;
	$conds = [] if ! defined $conds;

	my $wid = $what.'.'.$what.'_ID';
	my $tmp = __get_tag_infos($dbh, $what, $tags, [$wid], $conds, ["count($wid)"]);
	return FDN::Sql::Basic::__return_one_col($tmp,"count($wid)");
}

sub __get_tagged_n($$$;$$$$) {
	my ( $dbh, $what, $tags, $owner, $elems, $conds, $options ) = @_;
	my $tids = __resolve_tag_names($dbh, $tags, $owner);
	return __get_tagged($dbh, $what, $tids, $elems, $conds, $options);
}

sub __get_tagged($$$;$$$) {
	my ( $dbh, $what, $tags, $elems, $conds, $options ) = @_;
	$elems = [] if ! defined $elems;
	$conds = [] if ! defined $conds;
	$options = [] if ! defined $options;
	unshift @$elems, $what;

	my $tmp = __get_tag_infos($dbh, $what, $tags, $elems, $conds, $options);
	return FDN::Sql::Basic::__return_all_or_one($tmp);
}

sub __del_tags($$$) {
	my ( $dbh, $what, $whatid ) = @_;
	my $tagwhat = 'TAG'.$what;
	my $tmp = del_data($dbh, [$tagwhat], [$tagwhat.'.'.$what.'_ID='.dochar($whatid)]);
}

sub __set_tags_n($$$$;$$) {
	my ( $dbh, $what, $whatid, $tags, $owner, $create ) = @_;
	my $tids = __resolve_tag_names($dbh,$tags, $owner, $create );
	__set_tags($dbh, $what, $whatid, $tids);
}

sub __set_tags($$$$) {
	my ( $dbh, $what, $whatid, $tags ) = @_;
	for my $tag ( @$tags ) {
		__set_relation($dbh, 'TAG', $what, dochar($tag), dochar($whatid));
	}
}

sub __update_tags_n($$$$;$$) {
	my ( $dbh, $what, $whatid, $tags, $owner, $create ) = @_;
	__del_tags($dbh, $what, $whatid);
	__set_tags_n($dbh, $what, $whatid, $tags, $owner, $create);
}

sub __update_tags($$$$) {
	my ( $dbh, $what, $whatid, $tags ) = @_;
	__del_tags($dbh, $what, $whatid);
	__set_tags($dbh, $what, $whatid, $tags);
}

1;

