# This file is part of the "FDN tools" project.
# You can use and redistribute it under the terms of the
# GNU General Public License.
# (c) French Data Network 1997-2005
# Authors and contributors:
# - Benjamin Bayart
package FDN::Sql::Workflow::Basic;

use strict;
use Exporter ;

our @ISA = ( 'Exporter' ) ;

our @EXPORT = qw (
	workflow_select_db
	workflow_close_db

	new_task
	task_par_id
	task_par_key
	task_par_name
	task_par_name_state
	task_par_name_not_state
	task_par_name_proptask
	task_id_par_key
	set_task_nbchild
	set_task_state
	set_task_key
	set_task_properties
	get_task_properties
	del_task_properties
	update_task_properties
	del_task

	taskself_par_parent
	taskself_par_parent_numero
	taskself_par_fils
	taskself_par_fils_numero
	taskself_par_parent_fils
	new_taskself
	del_taskself
	 
	new_event
	event_par_id
	event_par_event
	set_event_properties
	get_event_properties
	del_event_properties
	update_event_properties
	del_event
	del_event_par_event
	purge_event
	purge_event_par_event
	lock_event
	unlock_event
	 
	new_waiting
	waiting_par_id
	waiting_par_event
	waiting_par_task
	del_waiting
	tous_waiting
	 
	new_run
	run_par_id
	run_par_task
	set_run_state
	run_par_state
	del_run
	lock_run
	unlock_run
);

use FDN::Common::Constantes;
use FDN::Common::Logger;
use FDN::Sql::DB_Access;
use FDN::Sql::Basic;
use FDN::Sql::DBC;

my $dbh;
our $debug = 0;

sub __debug($) {
	my ( $msg ) = @_;
	return if ! $debug;
	debug $msg;
}

sub workflow_select_db($) { # Test
	my ( $db ) = @_;
	# It should not be our purpose to close the connexion...?
	$dbh = do_connect($db);
	return $dbh;
}

sub workflow_close_db() { # Test
	do_disconnect($dbh);
}

# Tasks
sub new_task($$$) { # Test
	my ( $type, $name, $key ) = @_;
	return __new($dbh,'TASK',
			'TASK_TYPE='.dochar($type),
			'TASK_KEY='.dochar($key),
			'TASK_NAME='.dochar($name));
}

sub task_par_id($) { # Test
	return __par_id($dbh,'TASK',$_[0]);
}

sub task_par_key($) { # Test
	return __par_clef($dbh,'TASK',$_[0],'TASK_KEY');
}

sub task_par_name($) { # Test
	return __par_clef($dbh,'TASK',$_[0],'TASK_NAME');
}

sub task_par_name_state($$) { # Test
	my ( $name, $state ) = @_;
	return __par_champ_ext($dbh,'TASK', [
		[ 'TASK_NAME',  '=', dochar($name)  ],
		[ 'TASK_STATE', '=', dochar($state) ] ]);
}

sub task_par_name_not_state($$) { # Test
	my ( $name, $state ) = @_;
	return __par_champ_ext($dbh,'TASK', [
		[ 'TASK_NAME',   '=', dochar($name) ],
		[ 'TASK_STATE', '!=', dochar($state) ] ]);
}

sub task_par_name_proptask ($$$) { # Test
	my ($name, $nom, $val) = @_;
	return __par_champ_ext($dbh,'TASK',
			[ [ 'TASK_NAME', '=', dochar($name) ],
			  [ 'PROPTASK_NOM', '=', dochar($nom) ],
			  [ 'PROPTASK_VAL', '=', dochar($val) ],
			  [ 'PROPTASK.TASK_ID', '=', 'TASK.TASK_ID' ]],
			  { 'misc' => 'distinct' });
}

sub task_id_par_key($) { # Test
	my ( $key ) = @_;
	return __id_par_nom($dbh, 'TASK', $key, '=', 'TASK_KEY');
}

sub set_task_state($$) { # Test
	my ( $id, $state ) = @_;
	return __set_tbl_val($dbh,'TASK','TASK_STATE',$id,dochar($state));
}

sub set_task_key($$) { # Test
	my ( $id, $key ) = @_;
	return __set_tbl_val($dbh,'TASK','TASK_KEY',$id,dochar($key));
}

sub set_task_nbchild($$) { # Test
	my ( $id, $nb ) = @_;
	return __set_tbl_val($dbh,'TASK','TASK_NBCHILD',$id,$nb);
}

sub set_task_properties($$) { # Test
	my ( $id, $props ) = @_;
	return __set_properties($dbh,'TASK',$id,$props);
}

sub get_task_properties($) { # Test
	my ( $id ) = @_;
	return __get_properties($dbh,'TASK',$id);
}

sub del_task_properties($) { # Test
	my ( $id ) = @_;
	return __del_properties($dbh,'TASK',$id);
}

sub update_task_properties($$) { # Test
	my ( $id, $props ) = @_;
	return __update_properties($dbh,'TASK',$id,$props);
}

sub del_task($) { # Test
	my ( $id ) = @_;
	return __del($dbh,'TASK',$id);
}

# Taskself

sub taskself_par_parent($) { # Test
	return __get_self_relation($dbh,'TASK','TASK',$_[0]);
}

sub taskself_par_parent_numero($$) { # Test
	my ( $parent, $numero ) = @_;
	return __get_self_relation($dbh,'TASK','TASK',$parent,"ORDER=".$numero);
}

sub taskself_par_fils($) { # Test
	return __get_self_relation($dbh,'TASK','SELF',$_[0]);
}

sub taskself_par_fils_numero($$) { # Test
	my ( $fils, $numero ) = @_ ;
	return __get_self_relation($dbh,'TASK','SELF',$fils,"ORDER=".$numero);
}

sub taskself_par_parent_fils($$) { # Test
	my ( $parent, $fils ) = @_;
	return __get_self_relation($dbh,'TASK','TASK',$parent,"TASKSELF.SELF_ID=".$fils);
}

sub new_taskself($$$) { # Test
	my ( $parent, $fils, $numero ) = @_;
	return __set_self_relation($dbh,'TASK',$parent,$fils,"ORDER=".$numero);
}

sub del_taskself($$) { # Test
	my ( $parent, $fils ) = @_;
	return __del_relation($dbh,'TASK','TASK',$parent,$fils);
}

# Events

sub new_event($) { # Test
	my ( $event ) = @_;
	return __new($dbh,'EVENT', 'EVENT_EVENT='.dochar($event));
}

sub event_par_id($) { # Test
	my ( $id ) = @_;
	return __par_id($dbh,'EVENT',$id);
}

sub event_par_event($) { # Test
	my ( $event ) = @_;
	return __par_clef($dbh,'EVENT',$event,'EVENT_EVENT');
}

sub set_event_properties($$) { # Test
	my ( $id, $props ) = @_;
	return __set_properties($dbh,'EVENT',$id,$props);
}

sub get_event_properties($) { # Test
	my ( $id ) = @_;
	return __get_properties($dbh,'EVENT',$id);
}

sub update_event_properties($$) { # Test
	my ( $id, $props ) = @_;
	return __update_properties($dbh,'EVENT',$id,$props);
}

sub del_event_properties($) { # Test
	my ( $id ) = @_;
	return __del_properties($dbh,'EVENT',$id);
}

sub del_event($) { # Test
	my ( $id ) = @_;
	return __del($dbh,'EVENT',$id);
}

sub del_event_par_event($) { # Test
	my ( $event ) = @_;
	return __del_par_champ($dbh,'EVENT','EVENT',dochar($event));
}

sub purge_event($) { # Test
	my ( $id ) = @_;
	del_event_properties($id);
	del_event($id);
}

sub purge_event_par_event($) { # Test
	my ( $name ) = @_;
	my $event = event_par_event($name);
	my $id = $event->{EVENT_ID};
	del_event_properties($id);
	del_event($id);
}

sub lock_event() { # Test
	return lock_soft($dbh,'event');
}

sub unlock_event() { # Test
	return unlock_soft($dbh, 'event');
}

# Waitings

sub new_waiting($$) { # Test
	my ( $id, $event ) = @_;
	return __new($dbh,'WAITING',
			'TASK_ID='.$id,
			'WAITING_EVENT='.dochar($event));
}

sub waiting_par_id($) { # Test
	return __par_id($dbh,'WAITING',$_[0]);
}

sub waiting_par_event($) { # Test
	my ( $event ) = @_;
	return __par_clef($dbh,'WAITING',$event,'WAITING_EVENT');
}

sub waiting_par_task($) { # Test
	my ( $task ) = @_;
	return __par_clef($dbh,'WAITING',$task,'TASK_ID');
}

sub del_waiting($) { # Test
	my ( $id ) = @_;
	return __del($dbh,'WAITING',$id);
}

sub tous_waiting() { # Test
	return __tous($dbh,'WAITING');
}

# Runs

sub new_run($$) { # Test
	my ( $id, $event ) = @_;
	return __new($dbh,'RUN',
			'TASK_ID='.$id,
			'RUN_EVENT='.dochar($event),
			'RUN_STATE=\'free\'');
}

sub run_par_id($) { # Test
	# Aussi surprenant que ca paraisse, cette fonction ne sert QUE pour
	# les tests !
	return __par_id($dbh,'RUN',$_[0]);
}

sub run_par_task($) { # Test
	my ( $task ) = @_;
	return __par_clef($dbh,'RUN',$task,'TASK_ID');
}

sub run_par_state($) { # Test
	my ( $etat ) = @_;
	# Ca passe par __par_champ_ext, et ca pose donc une limite 1 dans le cas
	# normal.
	return __par_champ_test($dbh,'RUN',dochar($etat),'RUN_STATE','=');
}

sub del_run($) { # Test
	my ( $id ) = @_;
	return __del($dbh,'RUN',$id);
}

sub set_run_state($$) { # Test
	my ( $id, $state ) = @_;
	return __set_tbl_val($dbh,'RUN','RUN_STATE',$id,dochar($state));
}

sub lock_run() { # Test
	return lock_soft($dbh,'run');
}

sub unlock_run() { # Test
	return unlock_soft($dbh,'run');
}

TRUE;

